@@ -21,14 +21,9 @@
 # along with PyX; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-from pyx import bbox, canvasitem, deco, path, pswriter, pdfwriter, trafo, unit
-import t1file
-
-try:
-    set()
-except NameError:
-    # Python 2.3
-    from sets import Set as set
+import warnings
+from pyx import bbox, canvasitem, deco, path, pswriter, pdfwriter, trafo, unit, pycompat
+import t1file, afmfile
 
 
 ##############################################################################
@@ -44,8 +39,8 @@ class PST1file(pswriter.PSresource):
         self.type = "t1file"
         self.t1file = t1file
         self.id = t1file.name
-        self.glyphnames = set(glyphnames)
-        self.charcodes = set(charcodes)
+        self.glyphnames = pycompat.set(glyphnames)
+        self.charcodes = pycompat.set(charcodes)
 
     def merge(self, other):
         self.glyphnames.update(other.glyphnames)
@@ -167,7 +162,7 @@ class PDFfont(pdfwriter.PDFobject):
 
         self.fontname = fontname
         self.basefontname = basefontname
-        self.charcodes = set(charcodes)
+        self.charcodes = pycompat.set(charcodes)
         self.fontdescriptor = fontdescriptor
         self.encoding = encoding
         self.metric = metric
@@ -268,8 +263,8 @@ class PDFfontfile(pdfwriter.PDFobject):
     def __init__(self, t1file, glyphnames, charcodes):
         pdfwriter.PDFobject.__init__(self, "fontfile", t1file.name)
         self.t1file = t1file
-        self.glyphnames = set(glyphnames)
-        self.charcodes = set(charcodes)
+        self.glyphnames = pycompat.set(glyphnames)
+        self.charcodes = pycompat.set(charcodes)
 
     def merge(self, other):
         self.glyphnames.update(other.glyphnames)
@@ -324,7 +319,7 @@ class font:
 
 class T1font(font):
 
-    def __init__(self, t1file, metric):
+    def __init__(self, t1file, metric=None):
         self.t1file = t1file
         self.name = t1file.name
         self.metric = metric
@@ -364,7 +359,7 @@ class text_pt(canvasitem.canvasitem):
 
 class T1text_pt(text_pt):
 
-    def __init__(self, font, x_pt, y_pt, charcodes, size_pt, decoding=None, slant=None, ignorebbox=False, kerning=False, ligatures=False, spaced_pt=0):
+    def __init__(self, font, x_pt, y_pt, charcodes, size_pt, decoding=afmfile.unicodestring, slant=None, ignorebbox=False, kerning=False, ligatures=False, spaced_pt=0):
         if decoding is not None:
             self.glyphnames = [decoding[character] for character in charcodes]
             self.decode = True
@@ -380,6 +375,7 @@ class T1text_pt(text_pt):
         self.kerning = kerning
         self.ligatures = ligatures
         self.spaced_pt = spaced_pt
+        self._textpath = None
 
         if self.kerning and not self.decode:
             raise ValueError("decoding required for font metric access (kerning)")
@@ -390,7 +386,8 @@ class T1text_pt(text_pt):
 
     def bbox(self):
         if self.font.metric is None:
-            raise ValueError("metric missing")
+            warnings.warn("We are about to extract the bounding box from the path of the text. This is slow and differs from the font metric information. You should provide an afm file whenever possible.")
+            return self.textpath().bbox()
         if not self.decode:
             raise ValueError("decoding required for font metric access (bbox)")
         return bbox.bbox_pt(self.x_pt,
@@ -402,7 +399,7 @@ class T1text_pt(text_pt):
         """returns the name of the encoding (in encodings) mapping self.glyphnames to codepoints
         If no such encoding can be found or extended, a new encoding is added to encodings
         """
-        glyphnames = set(self.glyphnames)
+        glyphnames = pycompat.set(self.glyphnames)
         if len(glyphnames) > 256:
             raise ValueError("glyphs do not fit into one single encoding")
         for encodingname, encoding in encodings.items():
@@ -421,11 +418,8 @@ class T1text_pt(text_pt):
         encodings[encodingname] = dict([(glyphname, i) for i, glyphname in enumerate(glyphnames)])
         return encodingname
 
-    def processPS(self, file, writer, context, registry, bbox):
-        if not self.ignorebbox:
-            bbox += self.bbox()
-
-        if writer.text_as_path:
+    def textpath(self):
+        if self._textpath is None:
             if self.decode:
                 if self.kerning:
                     data = self.font.metric.resolvekernings(self.glyphnames, self.size_pt)
@@ -433,7 +427,7 @@ class T1text_pt(text_pt):
                     data = self.glyphnames
             else:
                 data = self.charcodes
-            textpath = path.path()
+            self._textpath = path.path()
             x_pt = self.x_pt
             y_pt = self.y_pt
             for i, value in enumerate(data):
@@ -443,11 +437,18 @@ class T1text_pt(text_pt):
                 else:
                     if i:
                         x_pt += self.spaced_pt
-                    glyphpath, wx_pt, wy_pt = self.font.t1file.getglyphpathwxwy_pt(value, self.size_pt, convertcharcode=not self.decode)
-                    textpath += glyphpath.transformed(trafo.translate_pt(x_pt, y_pt))
-                    x_pt += wx_pt
-                    y_pt += wy_pt
-            deco.decoratedpath(textpath, fillstyles=[]).processPS(file, writer, context, registry, bbox)
+                    glyphpath = self.font.t1file.getglyphpath_pt(x_pt, y_pt, value, self.size_pt, convertcharcode=not self.decode)
+                    self._textpath += glyphpath.path
+                    x_pt += glyphpath.wx_pt
+                    y_pt += glyphpath.wy_pt
+        return self._textpath
+
+    def processPS(self, file, writer, context, registry, bbox):
+        if not self.ignorebbox:
+            bbox += self.bbox()
+
+        if writer.text_as_path:
+            deco.decoratedpath(self.textpath(), fillstyles=[]).processPS(file, writer, context, registry, bbox)
         else:
             # register resources
             if self.font.t1file is not None:
@@ -466,7 +467,9 @@ class T1text_pt(text_pt):
                 fontname = newfontname
 
             if self.slant:
-                newfontmatrix = trafo.trafo_pt(matrix=((1, self.slant), (0, 1))) * self.font.t1file.fontmatrix
+                newfontmatrix = trafo.trafo_pt(matrix=((1, self.slant), (0, 1)))
+                if self.font.t1file is not None:
+                    newfontmatrix = newfontmatrix * self.font.t1file.fontmatrix
                 newfontname = "%s-slant%f" % (fontname, self.slant)
                 registry.add(_ChangeFontMatrix)
                 registry.add(PSchangefontmatrix(fontname, newfontname, newfontmatrix))
@@ -508,28 +511,7 @@ class T1text_pt(text_pt):
             bbox += self.bbox()
 
         if writer.text_as_path:
-            if self.decode:
-                if self.kerning:
-                    data = self.font.metric.resolvekernings(self.glyphnames, self.size_pt)
-                else:
-                    data = self.glyphnames
-            else:
-                data = self.charcodes
-            textpath = path.path()
-            x_pt = self.x_pt
-            y_pt = self.y_pt
-            for i, value in enumerate(data):
-                if self.kerning and i % 2:
-                    if value is not None:
-                        x_pt += value
-                else:
-                    if i:
-                        x_pt += self.spaced_pt
-                    glyphpath, wx_pt, wy_pt = self.font.t1file.getglyphpathwxwy_pt(value, self.size_pt, convertcharcode=not self.decode)
-                    textpath += glyphpath.transformed(trafo.translate_pt(x_pt, y_pt))
-                    x_pt += wx_pt
-                    y_pt += wy_pt
-            deco.decoratedpath(textpath, fillstyles=[]).processPDF(file, writer, context, registry, bbox)
+            deco.decoratedpath(self.textpath(), fillstyles=[]).processPDF(file, writer, context, registry, bbox)
         else:
             if self.decode:
                 encodingname = self.getencodingname(writer.encodings.setdefault(self.font.name, {}))
