@@ -1,9 +1,9 @@
 # -*- encoding: utf-8 -*-
 #
 #
-# Copyright (C) 2002-2004 Jörg Lehmann <joergl@users.sourceforge.net>
+# Copyright (C) 2002-2012 Jörg Lehmann <joergl@users.sourceforge.net>
 # Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
-# Copyright (C) 2002-2011 André Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2012 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -23,14 +23,33 @@
 
 
 import math, re, string, warnings
-from pyx import canvas, path, trafo, unit
-from pyx.graph import style
+from pyx import canvas, path, pycompat, trafo, unit
 from pyx.graph.axis import axis, positioner
 
 
 goldenmean = 0.5 * (math.sqrt(5) + 1)
 
 
+# The following two methods are used to register and get a default provider
+# for keys. A key is a variable name in sharedata. A provider is a style
+# which creates variables in sharedata.
+
+_defaultprovider = {}
+
+def registerdefaultprovider(style, keys):
+    """sets a style as a default creator for sharedata variables 'keys'"""
+    for key in keys:
+        assert key in style.providesdata, "key not provided by style"
+        # we might allow for overwriting the defaults, i.e. the following is not checked:
+        # assert key in _defaultprovider.keys(), "default provider already registered for key"
+        _defaultprovider[key] = style
+
+def getdefaultprovider(key):
+    """returns a style, which acts as a default creator for the
+    sharedata variable 'key'"""
+    return _defaultprovider[key]
+
+
 class styledata:
     """style data storage class
 
@@ -57,7 +76,7 @@ class plotitem:
             for s in styles:
                 for n in s.needsdata:
                     if n not in provided:
-                        defaultprovider = style.getdefaultprovider(n)
+                        defaultprovider = getdefaultprovider(n)
                         addstyles.append(defaultprovider)
                         provided.extend(defaultprovider.providesdata)
                 provided.extend(s.providesdata)
@@ -65,12 +84,13 @@ class plotitem:
 
         self.styles = styles
         self.sharedata = styledata()
+        self.dataaxisnames = {}
         self.privatedatalist = [styledata() for s in self.styles]
 
         # perform setcolumns to all styles
-        self.usedcolumnnames = []
+        self.usedcolumnnames = pycompat.set()
         for privatedata, s in zip(self.privatedatalist, self.styles):
-            self.usedcolumnnames.extend(s.columnnames(privatedata, self.sharedata, graph, self.data.columnnames))
+            self.usedcolumnnames.update(pycompat.set(s.columnnames(privatedata, self.sharedata, graph, self.data.columnnames, self.dataaxisnames)))
 
     def selectstyles(self, graph, selectindex, selecttotal):
         for privatedata, style in zip(self.privatedatalist, self.styles):
@@ -79,31 +99,39 @@ class plotitem:
     def adjustaxesstatic(self, graph):
         for columnname, data in self.data.columns.items():
             for privatedata, style in zip(self.privatedatalist, self.styles):
-                style.adjustaxis(privatedata, self.sharedata, graph, columnname, data)
+                style.adjustaxis(privatedata, self.sharedata, graph, self, columnname, data)
 
     def makedynamicdata(self, graph):
-        self.dynamiccolumns = self.data.dynamiccolumns(graph)
+        self.dynamiccolumns = self.data.dynamiccolumns(graph, self.dataaxisnames)
 
     def adjustaxesdynamic(self, graph):
         for columnname, data in self.dynamiccolumns.items():
             for privatedata, style in zip(self.privatedatalist, self.styles):
-                style.adjustaxis(privatedata, self.sharedata, graph, columnname, data)
+                style.adjustaxis(privatedata, self.sharedata, graph, self, columnname, data)
 
     def draw(self, graph):
         for privatedata, style in zip(self.privatedatalist, self.styles):
             style.initdrawpoints(privatedata, self.sharedata, graph)
-        point = {}
-        useitems = []
-        for columnname in self.usedcolumnnames:
-            try:
-                useitems.append((columnname, self.dynamiccolumns[columnname]))
-            except KeyError:
-                useitems.append((columnname, self.data.columns[columnname]))
-        if not useitems:
-            raise ValueError("cannot draw empty data")
-        for i in xrange(len(useitems[0][1])):
-            for columnname, data in useitems:
-                point[columnname] = data[i]
+
+        point = dict([(columnname, None) for columnname in self.usedcolumnnames])
+        # fill point with (static) column data first
+        columns = self.data.columns.keys()
+        for values in zip(*self.data.columns.values()):
+            for column, value in zip(columns, values):
+                point[column] = value
+            for privatedata, style in zip(self.privatedatalist, self.styles):
+                style.drawpoint(privatedata, self.sharedata, graph, point)
+
+        point = dict([(columnname, None) for columnname in self.usedcolumnnames])
+        # insert an empty point
+        if self.data.columns and self.dynamiccolumns:
+            for privatedata, style in zip(self.privatedatalist, self.styles):
+                style.drawpoint(privatedata, self.sharedata, graph, point)
+        # fill point with dynamic column data
+        columns = self.dynamiccolumns.keys()
+        for values in zip(*self.dynamiccolumns.values()):
+            for key, value in zip(columns, values):
+                point[key] = value
             for privatedata, style in zip(self.privatedatalist, self.styles):
                 style.drawpoint(privatedata, self.sharedata, graph, point)
         for privatedata, style in zip(self.privatedatalist, self.styles):
@@ -115,6 +143,8 @@ class plotitem:
 
     def __getattr__(self, attr):
         # read only access to the styles privatedata
+        # this is just a convenience method
+        # use case: access the path of a the line style
         stylesdata = [getattr(styledata, attr)
                       for styledata in self.privatedatalist
                       if hasattr(styledata, attr)]
@@ -129,6 +159,8 @@ class graph(canvas.canvas):
 
     def __init__(self):
         canvas.canvas.__init__(self)
+        for name in ["background", "filldata", "axes.grid", "axis.baseline", "axis.ticks", "axis.labels", "axis.title", "data", "key"]:
+            self.layer(name)
         self.axes = {}
         self.plotitems = []
         self.keyitems = []
@@ -149,6 +181,7 @@ class graph(canvas.canvas):
         self.finish()
         return canvas.canvas.bbox(self)
 
+
     def registerPS(self, registry):
         self.finish()
         canvas.canvas.registerPS(self, registry)
@@ -283,7 +316,7 @@ class graph(canvas.canvas):
 class graphxy(graph):
 
     def __init__(self, xpos=0, ypos=0, width=None, height=None, ratio=goldenmean,
-                 key=None, backgroundattrs=None, axesdist=0.8*unit.v_cm,
+                 key=None, backgroundattrs=None, axesdist=0.8*unit.v_cm, flipped=False,
                  xaxisat=None, yaxisat=None, **axes):
         graph.__init__(self)
 
@@ -296,6 +329,7 @@ class graphxy(graph):
         self.key = key
         self.backgroundattrs = backgroundattrs
         self.axesdist_pt = unit.topt(axesdist)
+        self.flipped = flipped
 
         self.width = width
         self.height = height
@@ -360,28 +394,57 @@ class graphxy(graph):
                 self.axesnames[1].append(axisname)
             aaxis.setcreatecall(self.doaxiscreate, axisname)
 
+        self.axespositioners = dict(x=positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt,
+                                                                self.xpos_pt + self.width_pt, self.ypos_pt,
+                                                                (0, 1), self.xvgridpath),
+                                    x2=positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt + self.height_pt,
+                                                                 self.xpos_pt + self.width_pt, self.ypos_pt + self.height_pt,
+                                                                 (0, -1), self.xvgridpath),
+                                    y=positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt,
+                                                                self.xpos_pt, self.ypos_pt + self.height_pt,
+                                                                (1, 0), self.yvgridpath),
+                                    y2=positioner.lineaxispos_pt(self.xpos_pt + self.width_pt, self.ypos_pt,
+                                                                 self.xpos_pt + self.width_pt, self.ypos_pt + self.height_pt,
+                                                                 (-1, 0), self.yvgridpath))
+        if self.flipped:
+            self.axespositioners = dict(x=self.axespositioners["y2"],
+                                        y2=self.axespositioners["x2"],
+                                        y=self.axespositioners["x"],
+                                        x2=self.axespositioners["y"])
 
     def pos_pt(self, x, y, xaxis=None, yaxis=None):
         if xaxis is None:
             xaxis = self.axes["x"]
         if yaxis is None:
             yaxis = self.axes["y"]
-        return (self.xpos_pt + xaxis.convert(x)*self.width_pt,
-                self.ypos_pt + yaxis.convert(y)*self.height_pt)
+        vx = xaxis.convert(x)
+        vy = yaxis.convert(y)
+        if self.flipped:
+            vx, vy = vy, vx
+        return (self.xpos_pt + vx*self.width_pt,
+                self.ypos_pt + vy*self.height_pt)
 
     def pos(self, x, y, xaxis=None, yaxis=None):
         if xaxis is None:
             xaxis = self.axes["x"]
         if yaxis is None:
             yaxis = self.axes["y"]
-        return (self.xpos + xaxis.convert(x)*self.width,
-                self.ypos + yaxis.convert(y)*self.height)
+        vx = xaxis.convert(x)
+        vy = yaxis.convert(y)
+        if self.flipped:
+            vx, vy = vy, vx
+        return (self.xpos + vx*self.width,
+                self.ypos + vy*self.height)
 
     def vpos_pt(self, vx, vy):
+        if self.flipped:
+            vx, vy = vy, vx
         return (self.xpos_pt + vx*self.width_pt,
                 self.ypos_pt + vy*self.height_pt)
 
     def vpos(self, vx, vy):
+        if self.flipped:
+            vx, vy = vy, vx
         return (self.xpos + vx*self.width,
                 self.ypos + vy*self.height)
 
@@ -393,6 +456,9 @@ class graphxy(graph):
 
     def vgeodesic(self, vx1, vy1, vx2, vy2):
         """returns a geodesic path between two points in graph coordinates"""
+        if self.flipped:
+            vx1, vy1 = vy1, vx1
+            vx2, vy2 = vy2, vx2
         return path.line_pt(self.xpos_pt + vx1*self.width_pt,
                             self.ypos_pt + vy1*self.height_pt,
                             self.xpos_pt + vx2*self.width_pt,
@@ -400,12 +466,18 @@ class graphxy(graph):
 
     def vgeodesic_el(self, vx1, vy1, vx2, vy2):
         """returns a geodesic path element between two points in graph coordinates"""
+        if self.flipped:
+            vx1, vy1 = vy1, vx1
+            vx2, vy2 = vy2, vx2
         return path.lineto_pt(self.xpos_pt + vx2*self.width_pt,
                               self.ypos_pt + vy2*self.height_pt)
 
     def vcap_pt(self, coordinate, length_pt, vx, vy):
         """returns an error cap path for a given coordinate, lengths and
         point in graph coordinates"""
+        if self.flipped:
+            coordinate = 1-coordinate
+            vx, vy = vy, vx
         if coordinate == 0:
             return path.line_pt(self.xpos_pt + vx*self.width_pt - 0.5*length_pt,
                                 self.ypos_pt + vy*self.height_pt,
@@ -427,39 +499,52 @@ class graphxy(graph):
         return path.line_pt(self.xpos_pt, self.ypos_pt + vy*self.height_pt,
                             self.xpos_pt + self.width_pt, self.ypos_pt + vy*self.height_pt)
 
-    def axistrafo(self, axis, t):
-        c = canvas.canvas([t])
-        c.insert(axis.canvas)
-        axis.canvas = c
+    def autokeygraphattrs(self):
+        return dict(direction="vertical", length=self.height)
+
+    def autokeygraphtrafo(self, keygraph):
+        dependsonaxisnumber = None
+        if self.flipped:
+            dependsonaxisname = "x"
+        else:
+            dependsonaxisname = "y"
+        for axisname in self.axes:
+            if axisname[0] == dependsonaxisname:
+                if len(axisname) == 1:
+                    axisname += "1"
+                axisnumber = int(axisname[1:])
+                if not (axisnumber % 2) and not self.flipped or (axisnumber % 2) and self.flipped:
+                    if dependsonaxisnumber is None or dependsonaxisnumber < axisnumber:
+                        dependsonaxisnumber = axisnumber
+        if dependsonaxisnumber is None:
+            x_pt = self.xpos_pt + self.width_pt
+        else:
+            if dependsonaxisnumber > 1:
+                dependsonaxisname += str(dependsonaxisnumber)
+            self.doaxiscreate(dependsonaxisname)
+            x_pt = self.axes[dependsonaxisname].positioner.x1_pt + self.axes[dependsonaxisname].canvas.extent_pt
+        x_pt += self.axesdist_pt
+        return trafo.translate_pt(x_pt, self.ypos_pt)
 
     def axisatv(self, axis, v):
         if axis.positioner.fixtickdirection[0]:
             # it is a y-axis
-            self.axistrafo(axis, trafo.translate_pt(self.xpos_pt + v*self.width_pt - axis.positioner.x1_pt, 0))
+            t = trafo.translate_pt(self.xpos_pt + v*self.width_pt - axis.positioner.x1_pt, 0)
         else:
             # it is an x-axis
-            self.axistrafo(axis, trafo.translate_pt(0, self.ypos_pt + v*self.height_pt - axis.positioner.y1_pt))
+            t = trafo.translate_pt(0, self.ypos_pt + v*self.height_pt - axis.positioner.y1_pt)
+        c = canvas.canvas()
+        for layer, subcanvas in axis.canvas.layers.items():
+            c.layer(layer).insert(subcanvas, [t])
+        assert len(axis.canvas.layers) == len(axis.canvas.items), str(axis.canvas.items)
+        axis.canvas = c
 
     def doaxispositioner(self, axisname):
         if self.did(self.doaxispositioner, axisname):
             return
         self.doranges()
-        if axisname == "x":
-            self.axes["x"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt,
-                                                                   self.xpos_pt + self.width_pt, self.ypos_pt,
-                                                                   (0, 1), self.xvgridpath))
-        elif axisname == "x2":
-            self.axes["x2"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt + self.height_pt,
-                                                                    self.xpos_pt + self.width_pt, self.ypos_pt + self.height_pt,
-                                                                    (0, -1), self.xvgridpath))
-        elif axisname == "y":
-            self.axes["y"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, self.ypos_pt,
-                                                                   self.xpos_pt, self.ypos_pt + self.height_pt,
-                                                                   (1, 0), self.yvgridpath))
-        elif axisname == "y2":
-            self.axes["y2"].setpositioner(positioner.lineaxispos_pt(self.xpos_pt + self.width_pt, self.ypos_pt,
-                                                                    self.xpos_pt + self.width_pt, self.ypos_pt + self.height_pt,
-                                                                    (-1, 0), self.yvgridpath))
+        if axisname in ["x", "x2", "y", "y2"]:
+            self.axes[axisname].setpositioner(self.axespositioners[axisname])
         else:
             if axisname[1:] == "3":
                 dependsonaxisname = axisname[0]
@@ -467,7 +552,9 @@ class graphxy(graph):
                 dependsonaxisname = "%s%d" % (axisname[0], int(axisname[1:]) - 2)
             self.doaxiscreate(dependsonaxisname)
             sign = 2*(int(axisname[1:]) % 2) - 1
-            if axisname[0] == "x":
+            if axisname[0] == "x" and self.flipped:
+                sign = -sign
+            if axisname[0] == "x" and not self.flipped or axisname[0] == "y" and self.flipped:
                 y_pt = self.axes[dependsonaxisname].positioner.y1_pt - sign * (self.axes[dependsonaxisname].canvas.extent_pt + self.axesdist_pt)
                 self.axes[axisname].setpositioner(positioner.lineaxispos_pt(self.xpos_pt, y_pt,
                                                                             self.xpos_pt + self.width_pt, y_pt,
@@ -492,8 +579,8 @@ class graphxy(graph):
         if self.did(self.dobackground):
             return
         if self.backgroundattrs is not None:
-            self.draw(path.rect_pt(self.xpos_pt, self.ypos_pt, self.width_pt, self.height_pt),
-                      self.backgroundattrs)
+            self.layer("background").draw(path.rect_pt(self.xpos_pt, self.ypos_pt, self.width_pt, self.height_pt),
+                                          self.backgroundattrs)
 
     def doaxes(self):
         if self.did(self.doaxes):
@@ -501,7 +588,9 @@ class graphxy(graph):
         self.dolayout()
         self.dobackground()
         for axis in self.axes.values():
-            self.insert(axis.canvas)
+            for layer, canvas in axis.canvas.layers.items():
+                self.layer("axes.%s" % layer).insert(canvas)
+            assert len(axis.canvas.layers) == len(axis.canvas.items), str(axis.canvas.items)
 
     def dokey(self):
         if self.did(self.dokey):
@@ -523,10 +612,65 @@ class graphxy(graph):
                 y = parentchildalign(self.ypos_pt, self.ypos_pt+self.height_pt,
                                      bbox.lly_pt, bbox.ury_pt,
                                      self.key.vpos, unit.topt(self.key.vdist), self.key.vinside)
-                self.insert(c, [trafo.translate_pt(x, y)])
+                self.layer("key").insert(c, [trafo.translate_pt(x, y)])
 
 
-class graphxyz(graphxy):
+
+class graphx(graphxy):
+
+    def __init__(self, xpos=0, ypos=0, length=None, size=0.5*unit.v_cm, direction="vertical",
+                 key=None, backgroundattrs=None, axesdist=0.8*unit.v_cm, **axes):
+        for name in axes:
+            if not name.startswith("x"):
+                raise ValueError("Only x axes are allowed")
+        self.direction = direction
+        if self.direction == "vertical":
+            kwargsxy = dict(width=size, height=length, flipped=True)
+        elif self.direction == "horizontal":
+            kwargsxy = dict(width=length, height=size)
+        else:
+            raise ValueError("vertical or horizontal direction required")
+        kwargsxy.update(**axes)
+
+        graphxy.__init__(self, xpos=xpos, ypos=ypos, ratio=None, key=key, y=axis.lin(min=0, max=1, parter=None),
+                         backgroundattrs=backgroundattrs, axesdist=axesdist, **kwargsxy)
+
+    def pos_pt(self, x, xaxis=None):
+        return graphxy.pos_pt(self, x, 0.5, xaxis)
+
+    def pos(self, x, xaxis=None):
+        return graphxy.pos(self, x, 0.5, xaxis)
+
+    def vpos_pt(self, vx):
+        return graphxy.vpos_pt(self, vx, 0.5)
+
+    def vpos(self, vx):
+        return graphxy.vpos(self, vx, 0.5)
+
+    def vgeodesic(self, vx1, vx2):
+        return graphxy.vgeodesic(self, vx1, 0.5, vx2, 0.5)
+
+    def vgeodesic_el(self, vx1, vy1, vx2, vy2):
+        return graphxy.vgeodesic_el(self, vx1, 0.5, vx2, 0.5)
+
+    def vcap_pt(self, coordinate, length_pt, vx):
+        if coordinate == 0:
+            return graphxy.vcap_pt(self, coordinate, length_pt, vx, 0.5)
+        else:
+            raise ValueError("direction invalid")
+
+    def xvgridpath(self, vx):
+        return graphxy.xvgridpath(self, vx)
+
+    def yvgridpath(self, vy):
+        raise Exception("This method does not exist on a one dimensional graph.")
+
+    def axisatv(self, axis, v):
+        raise Exception("This method does not exist on a one dimensional graph.")
+
+
+
+class graphxyz(graph):
 
     class central:
 
@@ -606,9 +750,12 @@ class graphxyz(graphxy):
 
     def __init__(self, xpos=0, ypos=0, size=None,
                  xscale=1, yscale=1, zscale=1/goldenmean,
-                 projector=central(10, -30, 30), key=None,
+                 projector=central(10, -30, 30), axesdist=0.8*unit.v_cm, key=None,
                  **axes):
         graph.__init__(self)
+        self.layer("hiddenaxes", below=self.layer("filldata"))
+        for name in ["hiddenaxes.grid", "hiddenaxes.baseline", "hiddenaxes.ticks", "hiddenaxes.labels", "hiddenaxes.title"]:
+            self.layer(name)
 
         self.xpos = xpos
         self.ypos = ypos
@@ -620,12 +767,25 @@ class graphxyz(graphxy):
         self.yscale = yscale
         self.zscale = zscale
         self.projector = projector
+        self.axesdist_pt = unit.topt(axesdist)
         self.key = key
 
         self.xorder = projector.zindex(0, -1, 0) > projector.zindex(0, 1, 0) and 1 or 0
         self.yorder = projector.zindex(-1, 0, 0) > projector.zindex(1, 0, 0) and 1 or 0
         self.zindexscale = math.sqrt(xscale*xscale+yscale*yscale+zscale*zscale)
 
+        # the pXYshow attributes are booleans stating whether plane perpendicular to axis X
+        # at the virtual graph coordinate Y will be hidden by data or not. An axis is considered
+        # to be visible if one of the two planes it is part of is visible. Other axes are drawn
+        # in the hiddenaxes layer (i.e. layer group).
+        # TODO: Tick and grid visibility is treated like the axis visibility at the moment.
+        self.pz0show = self.vangle(0, 0, 0, 1, 0, 0, 1, 1, 0) > 0
+        self.pz1show = self.vangle(0, 0, 1, 0, 1, 1, 1, 1, 1) > 0
+        self.py0show = self.vangle(0, 0, 0, 0, 0, 1, 1, 0, 1) > 0
+        self.py1show = self.vangle(0, 1, 0, 1, 1, 0, 1, 1, 1) > 0
+        self.px0show = self.vangle(0, 0, 0, 0, 1, 0, 0, 1, 1) > 0
+        self.px1show = self.vangle(1, 0, 0, 1, 0, 1, 1, 1, 1) > 0
+
         for axisname, aaxis in axes.items():
             if aaxis is not None:
                 if not isinstance(aaxis, axis.linkedaxis):
@@ -641,6 +801,8 @@ class graphxyz(graphxy):
                         self.axes[okey] = axis.linkedaxis(self.axes[axisname], okey)
                 else:
                     self.axes[axisname] = axis.linkedaxis(self.axes[okey], axisname)
+            elif not axes.has_key(okey):
+                self.axes[okey] = axis.linkedaxis(self.axes[axisname], okey)
         if not axes.has_key("z"):
             self.axes["z"] = axis.anchoredaxis(axis.linear(), self.texrunner, "z")
 
@@ -809,58 +971,103 @@ class graphxyz(graphxy):
                          path.lineto_pt(*self.vpos_pt(0, 1, vz)),
                          path.closepath())
 
+    def autokeygraphattrs(self):
+        return dict(direction="vertical", length=self.size)
+
+    def autokeygraphtrafo(self, keygraph):
+        self.doaxes()
+        x_pt = self.layer("axes").bbox().right_pt() + self.axesdist_pt
+        y_pt = 0.5*(self.layer("axes").bbox().top_pt() + self.layer("axes").bbox().bottom_pt() - self.size_pt)
+        return trafo.translate_pt(x_pt, y_pt)
+
     def doaxispositioner(self, axisname):
         if self.did(self.doaxispositioner, axisname):
             return
         self.doranges()
         if axisname == "x":
-            self.axes["x"].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, self.xorder, 0),
-                                                                       lambda vx: self.vtickdirection(vx, self.xorder, 0, vx, 1-self.xorder, 0),
-                                                                       self.xvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, self.xorder, 0),
+                                                                            lambda vx: self.vtickdirection(vx, self.xorder, 0, vx, 1-self.xorder, 0),
+                                                                            self.xvgridpath))
+            if self.xorder:
+                self.axes[axisname].hidden = not self.py1show and not self.pz0show
+            else:
+                self.axes[axisname].hidden = not self.py0show and not self.pz0show
         elif axisname == "x2":
-            self.axes["x2"].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, 1-self.xorder, 0),
-                                                                        lambda vx: self.vtickdirection(vx, 1-self.xorder, 0, vx, self.xorder, 0),
-                                                                        self.xvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, 1-self.xorder, 0),
+                                                                            lambda vx: self.vtickdirection(vx, 1-self.xorder, 0, vx, self.xorder, 0),
+                                                                            self.xvgridpath))
+            if self.xorder:
+                self.axes[axisname].hidden = not self.py0show and not self.pz0show
+            else:
+                self.axes[axisname].hidden = not self.py1show and not self.pz0show
         elif axisname == "x3":
-            self.axes["x3"].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, self.xorder, 1),
-                                                                        lambda vx: self.vtickdirection(vx, self.xorder, 1, vx, 1-self.xorder, 1),
-                                                                        self.xvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, self.xorder, 1),
+                                                                            lambda vx: self.vtickdirection(vx, self.xorder, 1, vx, 1-self.xorder, 1),
+                                                                            self.xvgridpath))
+            if self.xorder:
+                self.axes[axisname].hidden = not self.py1show and not self.pz1show
+            else:
+                self.axes[axisname].hidden = not self.py0show and not self.pz1show
         elif axisname == "x4":
-            self.axes["x4"].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, 1-self.xorder, 1),
-                                                                        lambda vx: self.vtickdirection(vx, 1-self.xorder, 1, vx, self.xorder, 1),
-                                                                        self.xvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vx: self.vpos_pt(vx, 1-self.xorder, 1),
+                                                                            lambda vx: self.vtickdirection(vx, 1-self.xorder, 1, vx, self.xorder, 1),
+                                                                            self.xvgridpath))
+            if self.xorder:
+                self.axes[axisname].hidden = not self.py0show and not self.pz1show
+            else:
+                self.axes[axisname].hidden = not self.py1show and not self.pz1show
         elif axisname == "y":
-            self.axes["y"].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(self.yorder, vy, 0),
-                                                                       lambda vy: self.vtickdirection(self.yorder, vy, 0, 1-self.yorder, vy, 0),
-                                                                       self.yvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(self.yorder, vy, 0),
+                                                                            lambda vy: self.vtickdirection(self.yorder, vy, 0, 1-self.yorder, vy, 0),
+                                                                            self.yvgridpath))
+            if self.yorder:
+                self.axes[axisname].hidden = not self.px1show and not self.pz0show
+            else:
+                self.axes[axisname].hidden = not self.px0show and not self.pz0show
         elif axisname == "y2":
-            self.axes["y2"].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(1-self.yorder, vy, 0),
-                                                                       lambda vy: self.vtickdirection(1-self.yorder, vy, 0, self.yorder, vy, 0),
-                                                                       self.yvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(1-self.yorder, vy, 0),
+                                                                            lambda vy: self.vtickdirection(1-self.yorder, vy, 0, self.yorder, vy, 0),
+                                                                            self.yvgridpath))
+            if self.yorder:
+                self.axes[axisname].hidden = not self.px0show and not self.pz0show
+            else:
+                self.axes[axisname].hidden = not self.px1show and not self.pz0show
         elif axisname == "y3":
-            self.axes["y3"].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(self.yorder, vy, 1),
-                                                                       lambda vy: self.vtickdirection(self.yorder, vy, 1, 1-self.yorder, vy, 1),
-                                                                       self.yvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(self.yorder, vy, 1),
+                                                                            lambda vy: self.vtickdirection(self.yorder, vy, 1, 1-self.yorder, vy, 1),
+                                                                            self.yvgridpath))
+            if self.yorder:
+                self.axes[axisname].hidden = not self.px1show and not self.pz1show
+            else:
+                self.axes[axisname].hidden = not self.px0show and not self.pz1show
         elif axisname == "y4":
-            self.axes["y4"].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(1-self.yorder, vy, 1),
-                                                                       lambda vy: self.vtickdirection(1-self.yorder, vy, 1, self.yorder, vy, 1),
-                                                                       self.yvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vy: self.vpos_pt(1-self.yorder, vy, 1),
+                                                                            lambda vy: self.vtickdirection(1-self.yorder, vy, 1, self.yorder, vy, 1),
+                                                                            self.yvgridpath))
+            if self.yorder:
+                self.axes[axisname].hidden = not self.px0show and not self.pz1show
+            else:
+                self.axes[axisname].hidden = not self.px1show and not self.pz1show
         elif axisname == "z":
-            self.axes["z"].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(0, 0, vz),
-                                                                       lambda vz: self.vtickdirection(0, 0, vz, 1, 1, vz),
-                                                                       self.zvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(0, 0, vz),
+                                                                            lambda vz: self.vtickdirection(0, 0, vz, 1, 1, vz),
+                                                                            self.zvgridpath))
+            self.axes[axisname].hidden = not self.px0show and not self.py0show
         elif axisname == "z2":
-            self.axes["z2"].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(1, 0, vz),
-                                                                       lambda vz: self.vtickdirection(1, 0, vz, 0, 1, vz),
-                                                                       self.zvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(1, 0, vz),
+                                                                            lambda vz: self.vtickdirection(1, 0, vz, 0, 1, vz),
+                                                                            self.zvgridpath))
+            self.axes[axisname].hidden = not self.px1show and not self.py0show
         elif axisname == "z3":
-            self.axes["z3"].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(0, 1, vz),
-                                                                       lambda vz: self.vtickdirection(0, 1, vz, 1, 0, vz),
-                                                                       self.zvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(0, 1, vz),
+                                                                            lambda vz: self.vtickdirection(0, 1, vz, 1, 0, vz),
+                                                                            self.zvgridpath))
+            self.axes[axisname].hidden = not self.px0show and not self.py1show
         elif axisname == "z4":
-            self.axes["z4"].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(1, 1, vz),
-                                                                       lambda vz: self.vtickdirection(1, 1, vz, 0, 0, vz),
-                                                                       self.zvgridpath))
+            self.axes[axisname].setpositioner(positioner.flexlineaxispos_pt(lambda vz: self.vpos_pt(1, 1, vz),
+                                                                            lambda vz: self.vtickdirection(1, 1, vz, 0, 0, vz),
+                                                                            self.zvgridpath))
+            self.axes[axisname].hidden = not self.px1show and not self.py1show
         else:
             raise NotImplementedError("4 axis per dimension supported only")
 
@@ -880,7 +1087,10 @@ class graphxyz(graphxy):
         self.dolayout()
         self.dobackground()
         for axis in self.axes.values():
-            self.insert(axis.canvas)
+            if axis.hidden:
+                self.layer("hiddenaxes").insert(axis.canvas)
+            else:
+                self.layer("axes").insert(axis.canvas)
 
     def dokey(self):
         if self.did(self.dokey):
